#!/usr/bin/python
# $Header: /home/eliot/asi/repo/cvsrepo/Repository/drv/hpi/python/hpivolumes.py,v 1.10 2011/12/21 02:10:47 as-ewb Exp $
#
# Example use of python hpi bindings

from __future__ import print_function

import sys
import audioscience.hpi as hpi
from sys import exit
from Tkinter import *
from ScrolledText import ScrolledText
from time import *
from audioscience.scrolled import Scrolledframe
from collections import defaultdict
from hpimeters import HpiMeter

def parseArgs():
    """Parse the commandline arguments"""
    from optparse import OptionParser, OptionGroup

    parser = OptionParser()
    parser.add_option('-a','--adapter',type='int',dest='adapter',
                      help='Adapter index. Default=%default', default=0)

    parser.add_option('-n','--node',type='int',dest='node',
                      help='Node type=%default', default=0)

    parser.add_option('-I','--interface-ip-address',type='string',dest='ipaddr',
                          help='Interface IP address.', default=None)

    return parser.parse_args()

opts,args = parseArgs()
hpi.setup(opts.adapter, opts.ipaddr)

sstate = ('?','Stopped','Playing','Recording','Xrun')

hpi.NODE = {100:'none', 101:'Stream', 102:'Line', 103:'AES3',
    104:'Tuner', 105:'Rf', 106:'Clock', 107:'Bitstream', 108:'Microphone',
    109:'Cobranet', 110:'Analog', 111:'Adapter', 112:'RTP', 113:'Internal',
    200:'none',201:'Stream',202:'Line',203:'AES3',204:'Rf',
    205:'Speaker',206:'Cobranet', 207:'Analog', 208:'RTP'}

def AdapterInfoString(adapter):
  e,nos,nis,ver,serial,atype = hpi.Adapter_GetInfo(adapter)
  return "%d: ASI%x IS=%d OS=%d" % (adapter,atype, nis, nos)

class HpiVolume (Frame):

  def __init__(self, master, hmixer, node1, index1, node2=0, index2=0):
    self.hx = hmixer

    if (node2) :
        e,self.hv=hpi.Mixer_GetControl(self.hx,node1,index1,node2,index2,hpi.CONTROL_VOLUME)
    elif (node1 < 200):
        e,self.hv=hpi.Mixer_GetControl(self.hx,node1,index1,0,0,hpi.CONTROL_VOLUME)
    else:
        e,self.hv=hpi.Mixer_GetControl(self.hx,0,0,node1,index1,hpi.CONTROL_VOLUME)

    if e:
        raise LookupError('Volume on %s %d %s %d not found' % (hpi.SourceDict[node1],index1,hpi.DestDict[node2],index2))

    Frame.__init__(self, master, borderwidth=2, relief='groove')

    e,self.chans = hpi.Volume_QueryChannels(self.hv)
    if e or self.chans > 2:
        self.chans = 2

    r = hpi.Volume_QueryRange(self.hv)
    if r[0]:
        self.min = -100
        self.max = 0
    else:
        self.min = r[1] / hpi.UNITS_PER_dB
        self.max = r[2] / hpi.UNITS_PER_dB

    e,r = hpi.Volume_GetGain(self.hv)
    self.current = r

    self.left = IntVar()
    self.left.set(r[0] / hpi.UNITS_PER_dB)
    self.left.trace('w', self.onLeft)

    self.right = IntVar()
    self.right.set(r[1] / hpi.UNITS_PER_dB)
    self.right.trace('w', self.onRight)

    self.createWidgets()

  def createWidgets(self):
    self.ml = Scale(self)
    self.ml.pack(side='left')
    self.ml['from'] = self.max
    self.ml['to'] = self.min
    self.ml["variable"]=self.left

    if self.chans > 1:
        self.mr = Scale(self)
        self.mr['from'] = self.max
        self.mr['to'] = self.min
        self.mr["variable"]=self.right
        self.mr.pack(side='left')
    # registering callback
    self.listenID = self.after(1000, self.updateVol)

  def updateVol(self):
    self.listenID = self.after(250, self.updateVol)
    e,r = hpi.Volume_GetGain(self.hv)
    if r != self.current:
        self.current = r
        self.left.set(r[0] / hpi.UNITS_PER_dB)
        self.right.set(r[1] / hpi.UNITS_PER_dB)

  def onLeft(self,name,index,mode):
    v = self.left.get() * hpi.UNITS_PER_dB
    if v != self.current[0]:
        self.current = (int(v), self.current[1])
        hpi.Volume_SetGain(self.hv, self.current)

  def onRight(self,name,index,mode):
    v = self.right.get() * hpi.UNITS_PER_dB
    self.current = (self.current[0], int(v))
    hpi.Volume_SetGain(self.hv, self.current)


class HpiMux(Frame):

  def __init__(self, master, hmixer, node, index):
    if (node < 200):
        e,self.hm = hpi.Mixer_GetControl(hmixer, node, index, 0, 0, hpi.CONTROL_MULTIPLEXER)
    else:
        e,self.hm = hpi.Mixer_GetControl(hmixer, 0, 0, node, index, hpi.CONTROL_MULTIPLEXER)

    if e:
        raise LookupError('Mux on %d %d not found' % (node, index))

    Frame.__init__(self, master, borderwidth=2, relief='groove')

    e,cnode,cindex = hpi.Multiplexer_GetSource(self.hm)
    self.current = ('%s-%d' % (hpi.NODE[cnode], cindex), cnode, cindex)

    self.items = []
    for i in range(0,32):
        e,node,index = hpi.Multiplexer_QuerySource(self.hm, i)
        if e:
            break;
        item = ('%s-%d' % (hpi.NODE[node], index), node, index)
        self.items.append(item)

    self.var = StringVar(self)
    self.var.set(self.current[0])

    self.m = OptionMenu(self, self.var, *[i[0] for i in self.items])
    self.m.pack(side='left')

    # registering callback
    #self.listenID = self.after(1000, self.updateMux)


def control_matrix(master, hmixer, sources, dests):

    # Top corner labels
    Label(master, text='> Destinations >').grid(row=0, column=1)
    Label(master, text='v Sources v').grid(row=1, column=0)
    Label(master, text='Master').grid(row=1, column=2)

    row = 0
    col = 3
    # Top row labels
    for d in dests:
        for di in range(d[1]):
            Label(master, text='%s#%d' % (hpi.NODE[d[0]], di)).grid(row=row, column=col)
            HpiMeter(master, hmixer, d[0], di).grid(row=row+1, column=col)
            try:
                HpiMux(master, hmixer, d[0], di).grid(row=row+2, column=col)
            except LookupError:
                pass
            try:
                HpiVolume(master, hmixer, d[0], di).grid(row=row+3, column=col)
            except LookupError:
                pass
            col += 1
    row = 3
    for s in sources:
        for si in range(s[1]):
            row += 1
            col = 0

            Label(master, text='%s#%d' % (hpi.NODE[s[0]], si)).grid(row=row, column=col)
            col += 1

            try:
                HpiMeter(master, hmixer, s[0], si).grid(row=row, column=col)
            except LookupError:
                pass
            col += 1
            try:
                HpiMux(master, hmixer, s[0], si).grid(row=row, column=col)
            except LookupError:
                pass

            try:
                HpiVolume(master, hmixer, s[0], si).grid(row=row, column=col)
            except LookupError:
                pass

            col += 1
            for d in dests:
                for di in range(d[1]):
                    try:
                        HpiVolume(master, hmixer, s[0], si, d[0], di).grid(row=row, column=col)
                        col += 1
                    except LookupError:
                        continue


def count_nodes(hmixer, req_type=None):
    sd = defaultdict(int)
    dd = defaultdict(int)
    for ci in range(10000):
        e,srctype,srcindex,dsttype,dstindex,ctrltype,ch = hpi.Mixer_GetControlByIndex(hmixer, ci)
        if e:
            break
        if req_type and req_type != ctrltype:
            continue

        if srctype != hpi.SOURCENODE_NONE:
            if sd[srctype] < srcindex + 1:
                sd[srctype] = srcindex + 1
        if dsttype != hpi.DESTNODE_NONE:
            if dd[dsttype] < dstindex + 1:
                dd[dsttype] = dstindex + 1

    sd = list(sd.iteritems())
    sd.sort()
    dd = list(dd.iteritems())
    dd.sort()
    return sd, dd


if __name__ == "__main__":
    adapter = opts.adapter
    hpi.setup(opts.adapter)

    e = hpi.Adapter_Open(adapter)
    if e:
        sys.exit('HPI error %d' % e)

    e,nos,nis,ver,serial,atype = hpi.Adapter_GetInfo(adapter)
    e,amode = hpi.Adapter_GetMode(adapter)
    if not e:
        mode = hpi.AdapterModeDict[amode]
        mode = '\nMode:' + mode.lstrip('ADAPTER_MODE').lower()
    else:
        mode = ''

    e, hmixer = hpi.Mixer_Open(adapter)
    sd, dd = count_nodes(hmixer, hpi.CONTROL_VOLUME)
    sm, dm = count_nodes(hmixer, hpi.CONTROL_MULTIPLEXER)

    s = list(set(sd + sm))
    d = list(set(dd + dm))
    s.sort()
    d.sort()

    app = Tk()
    app.title("HPI Volume Controls")
    scrollframe = Scrolledframe(app,stretch=True)

    Label(scrollframe(), text='ASI%04X\n#%d%s' % (atype, serial, mode)).grid()

    control_matrix(scrollframe(), hmixer, s, dd+dm)

    scrollx = Scrollbar(app, orient="horizontal", command=scrollframe.xview)
    scrollx.grid(row=1, column=0, sticky="nwse")

    scrolly = Scrollbar(app, orient="vertical", command=scrollframe.yview)
    scrolly.grid(row=0, column=1, sticky="nwse")

    scrollframe["xscrollcommand"] = scrollx.set
    scrollframe["yscrollcommand"] = scrolly.set

    scrollframe.grid(row=0,column=0,sticky="nwse")
    #scrollframe.pack()

    app.columnconfigure(0,weight=1)
    app.rowconfigure(0,weight=1)
    app.geometry("1280x1024")
    app.mainloop()
############################################################################
